@@ -31,8 +31,8 @@ sub run {
     } else {
         $self->log(
             $subcommand . " wasn't understood by the 'UID' command" );
-        $self->no_failed(
-            alert => q{Your client sent a UID command we didn't understand} );
+        $self->bad_command(
+            q{Unrecognized UID command $subcommand} );
     }
 
 }
@@ -105,8 +105,6 @@ sub copy {
     my $sequence = join(",",map {$_->uid} @messages);
     my $uids     = join(",",map {$_->uid} @new);
     $self->ok_command("[COPYUID @{[$mailbox->uidvalidity]} $sequence $uids] COPY COMPLETED");
-
-    $self->ok_completed;
 }
 
 sub expunge {
